import * as stream from 'stream';
import decryptChunk from '../decryption/decryptChunk';

export default function decryptStream(inputStream: stream.Readable, [key0, key1, key2]: [number, number, number]): stream.Readable {
  let skippedRandomHeader = false;

  const outStream = new stream.Readable({
    read(size) {
      //There are 12 random bytes at the beginning, we need to use them to initialize the decryption keys, but we can ignore the decrypted bytes.
      if (!skippedRandomHeader) {
        const encryptedHeader: Buffer = inputStream.read(12);
        const { keys } = decryptChunk(encryptedHeader, [key0, key1, key2]);
        [key0, key1, key2] = keys;
        skippedRandomHeader = true;
      }

      //Decrypt chunk
      const encryptedChunk: Buffer = inputStream.read(size);
      if (encryptedChunk === null) {
        //If end has been reached, stop
        this.push(null);
      } else {
        const { decryptedChunk, keys } = decryptChunk(encryptedChunk, [key0, key1, key2]);
        [key0, key1, key2] = keys;
        this.push(decryptedChunk);
      }
    },
  });

  inputStream.on('end', () => {
    outStream.emit('end');
  });

  return outStream;
}
